"use strict";
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
Object.defineProperty(exports, "__esModule", { value: true });
exports.MigrationGeocoder = void 0;
const client_location_1 = require("@aws-sdk/client-location");
const googleCommon_1 = require("./googleCommon");
const places_1 = require("./places");
class MigrationGeocoder {
    geocode(request, callback) {
        const bounds = request.bounds;
        const language = request.language;
        const location = request.location;
        const address = request.address;
        const placeId = request.placeId;
        // These are the only fields we want to return for the GeocoderResult
        const fields = ["formatted_address", "geometry", "place_id"];
        if (location) {
            const lngLat = (0, googleCommon_1.LatLngToLngLat)(location);
            const input = {
                IndexName: this._placeIndexName,
                Position: lngLat,
            };
            if (language) {
                input.Language = language;
            }
            return new Promise((resolve, reject) => {
                const command = new client_location_1.SearchPlaceIndexForPositionCommand(input);
                this._client
                    .send(command)
                    .then((response) => {
                    const googleResults = [];
                    const results = response.Results;
                    if (results.length !== 0) {
                        results.forEach(function (place) {
                            const newPlace = (0, places_1.convertAmazonPlaceToGoogle)(place, fields, false);
                            googleResults.push(newPlace);
                        });
                    }
                    if (callback) {
                        callback(googleResults, googleCommon_1.GeocoderStatus.OK);
                    }
                    resolve({
                        results: googleResults,
                    });
                })
                    .catch((error) => {
                    console.error(error);
                    if (callback) {
                        callback(null, googleCommon_1.GeocoderStatus.UNKNOWN_ERROR);
                    }
                    reject({
                        status: googleCommon_1.GeocoderStatus.UNKNOWN_ERROR,
                    });
                });
            });
        }
        else if (placeId) {
            return new Promise((resolve, reject) => {
                const request = {
                    placeId: placeId,
                    fields: fields,
                };
                this._placesService.getDetails(request, (result, status) => {
                    if (status == googleCommon_1.PlacesServiceStatus.OK) {
                        const googleResults = [result];
                        if (callback) {
                            callback(googleResults, googleCommon_1.GeocoderStatus.OK);
                        }
                        resolve({
                            results: googleResults,
                        });
                    }
                    else {
                        if (callback) {
                            callback(null, googleCommon_1.GeocoderStatus.UNKNOWN_ERROR);
                        }
                        reject({
                            status: googleCommon_1.GeocoderStatus.UNKNOWN_ERROR,
                        });
                    }
                });
            });
        }
        else if (address) {
            return new Promise((resolve, reject) => {
                const request = {
                    query: address,
                    fields: fields,
                };
                // TODO: findPlaceFromQuery only supports a locationBias instead of a bounds, so if bounds
                // was specified we will just use the center to act as a locationBias
                if (bounds) {
                    const latLngBounds = new googleCommon_1.MigrationLatLngBounds(bounds);
                    request.locationBias = latLngBounds.getCenter();
                }
                this._placesService.findPlaceFromQuery(request, (results, status) => {
                    if (status == googleCommon_1.PlacesServiceStatus.OK) {
                        if (callback) {
                            callback(results, googleCommon_1.GeocoderStatus.OK);
                        }
                        resolve({
                            results: results,
                        });
                    }
                    else {
                        if (callback) {
                            callback(null, googleCommon_1.GeocoderStatus.UNKNOWN_ERROR);
                        }
                        reject({
                            status: googleCommon_1.GeocoderStatus.UNKNOWN_ERROR,
                        });
                    }
                });
            });
        }
    }
}
exports.MigrationGeocoder = MigrationGeocoder;
