"use strict";
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
Object.defineProperty(exports, "__esModule", { value: true });
const amazon_location_utilities_auth_helper_1 = require("@aws/amazon-location-utilities-auth-helper");
const client_location_1 = require("@aws-sdk/client-location");
const directions_1 = require("./directions");
const geocoder_1 = require("./geocoder");
const googleCommon_1 = require("./googleCommon");
const maps_1 = require("./maps");
const markers_1 = require("./markers");
const places_1 = require("./places");
const infoWindow_1 = require("./infoWindow");
const events_1 = require("./events");
const version_1 = require("./version");
// Dynamically load the MapLibre and MapLibre Geocoder stylesheets so that our migration SDK is the only thing our users need to import
// Without this, many MapLibre rendering features won't work (e.g. markers and info windows won't be visible)
// Also the MapLibre Geocoder input field won't function properly
const maplibreStyle = document.createElement("link");
maplibreStyle.setAttribute("rel", "stylesheet");
maplibreStyle.setAttribute("href", "https://unpkg.com/maplibre-gl@3.x/dist/maplibre-gl.css");
document.head.appendChild(maplibreStyle);
const maplibreGeocoderStyle = document.createElement("link");
maplibreGeocoderStyle.setAttribute("rel", "stylesheet");
maplibreGeocoderStyle.setAttribute("href", "https://www.unpkg.com/@aws/amazon-location-for-maplibre-gl-geocoder@1.x/dist/amazon-location-for-mlg-styles.css");
document.head.appendChild(maplibreGeocoderStyle);
// Parse URL params from the query string this script was imported with so we can retrieve
// params (e.g. API key, place index, etc...)
const currentScript = document.currentScript;
const currentScriptSrc = currentScript.src;
const queryString = currentScriptSrc.substring(currentScriptSrc.indexOf("?"));
const urlParams = new URLSearchParams(queryString);
// API key is required to be passed in, so if it's not we need to log an error and bail out
// TODO: Add cognito support as well
const apiKey = urlParams.get("apiKey");
// Optional, the region to be used (us-west-2 by default)
const defaultRegion = "us-west-2";
const region = urlParams.get("region") || defaultRegion;
// Optional, but if user wants to perform any Places requests, this is required
const placeIndexName = urlParams.get("placeIndex");
// Optional, but if user wants to perform any Route requests, this is required
const routeCalculatorName = urlParams.get("routeCalculator");
// Optional, will invoke after migrationInit has finished executing
const postMigrationCallback = urlParams.get("callback");
// Optional, but if user wants to use a Map, this is required
const mapName = urlParams.get("map");
// Style URL is used by the Map for making requests
const styleUrl = `https://maps.geo.${region}.amazonaws.com/maps/v0/maps/${mapName}/style-descriptor?key=${apiKey}`;
const migrationInit = async function () {
    // Pass our style url (which includes the API key) to our Migration Map class
    maps_1.MigrationMap.prototype._styleUrl = styleUrl;
    // Create an authentication helper instance using an API key
    const authHelper = await (0, amazon_location_utilities_auth_helper_1.withAPIKey)(apiKey);
    const client = new client_location_1.LocationClient(Object.assign({ region: region, customUserAgent: `migration-sdk-${version_1.PACKAGE_VERSION}` }, authHelper.getLocationClientConfig()));
    // Pass our location client, and optionally place index and route calculator names
    // to our migration services
    places_1.MigrationAutocomplete.prototype._client = client;
    places_1.MigrationAutocomplete.prototype._placeIndexName = placeIndexName;
    places_1.MigrationAutocompleteService.prototype._client = client;
    places_1.MigrationAutocompleteService.prototype._placeIndexName = placeIndexName;
    geocoder_1.MigrationGeocoder.prototype._client = client;
    geocoder_1.MigrationGeocoder.prototype._placeIndexName = placeIndexName;
    places_1.MigrationPlace._client = client;
    places_1.MigrationPlace._placeIndexName = placeIndexName;
    places_1.MigrationPlacesService.prototype._client = client;
    places_1.MigrationPlacesService.prototype._placeIndexName = placeIndexName;
    places_1.MigrationSearchBox.prototype._client = client;
    places_1.MigrationSearchBox.prototype._placeIndexName = placeIndexName;
    directions_1.MigrationDirectionsService.prototype._client = client;
    directions_1.MigrationDirectionsService.prototype._routeCalculatorName = routeCalculatorName;
    directions_1.MigrationDistanceMatrixService.prototype._client = client;
    directions_1.MigrationDistanceMatrixService.prototype._routeCalculatorName = routeCalculatorName;
    // Additionally, we need to create a places service for our directions service and distance matrix
    // service to use, since it can optionally be passed source/destinations that are string queries
    // instead of actual LatLng coordinates.
    // We also need to create a places service for the Geocoder, which will use it for the
    // address and placeId geocode requests.
    // Constructing it here and passing it in will make sure
    // it is already configured with the appropriate client and place index name.
    directions_1.MigrationDirectionsService.prototype._placesService = new places_1.MigrationPlacesService();
    directions_1.MigrationDistanceMatrixService.prototype._placesService = new places_1.MigrationPlacesService();
    geocoder_1.MigrationGeocoder.prototype._placesService = new places_1.MigrationPlacesService();
    // Create the Google Maps namespace with our migration classes
    // eslint-disable-next-line  @typescript-eslint/no-explicit-any
    window.google = {
        maps: {
            LatLng: googleCommon_1.MigrationLatLng,
            LatLngBounds: googleCommon_1.MigrationLatLngBounds,
            Map: maps_1.MigrationMap,
            Marker: markers_1.MigrationMarker,
            marker: {
                AdvancedMarkerElement: markers_1.MigrationMarker,
            },
            InfoWindow: infoWindow_1.MigrationInfoWindow,
            ControlPosition: googleCommon_1.MigrationControlPosition,
            DirectionsRenderer: directions_1.MigrationDirectionsRenderer,
            DirectionsService: directions_1.MigrationDirectionsService,
            DistanceMatrixService: directions_1.MigrationDistanceMatrixService,
            DirectionsStatus: googleCommon_1.DirectionsStatus,
            TravelMode: directions_1.TravelMode,
            UnitSystem: directions_1.UnitSystem,
            DistanceMatrixElementStatus: directions_1.DistanceMatrixElementStatus,
            DistanceMatrixStatus: directions_1.DistanceMatrixStatus,
            Geocoder: geocoder_1.MigrationGeocoder,
            GeocoderStatus: googleCommon_1.GeocoderStatus,
            places: {
                Autocomplete: places_1.MigrationAutocomplete,
                AutocompleteService: places_1.MigrationAutocompleteService,
                Place: places_1.MigrationPlace,
                PlacesService: places_1.MigrationPlacesService,
                PlacesServiceStatus: googleCommon_1.PlacesServiceStatus,
                SearchBox: places_1.MigrationSearchBox,
            },
            event: {
                addListener: events_1.addListener,
                addListenerOnce: events_1.addListenerOnce,
                removeListener: events_1.removeListener,
            },
            // Handle dynamic imports, e.g. const { Map } = await google.maps.importLibrary("maps");
            importLibrary: (library) => {
                return new Promise((resolve) => {
                    switch (library) {
                        case "core":
                            resolve({
                                ControlPosition: googleCommon_1.MigrationControlPosition,
                                LatLng: googleCommon_1.MigrationLatLng,
                                LatLngBounds: googleCommon_1.MigrationLatLngBounds,
                                event: {
                                    addListener: events_1.addListener,
                                    addListenerOnce: events_1.addListenerOnce,
                                    removeListener: events_1.removeListener,
                                },
                            });
                            break;
                        case "geocoding":
                            resolve({
                                Geocoder: geocoder_1.MigrationGeocoder,
                                GeocoderStatus: googleCommon_1.GeocoderStatus,
                            });
                            break;
                        case "maps":
                            resolve({
                                InfoWindow: infoWindow_1.MigrationInfoWindow,
                                Map: maps_1.MigrationMap,
                            });
                            break;
                        case "places":
                            resolve({
                                Autocomplete: places_1.MigrationAutocomplete,
                                AutocompleteService: places_1.MigrationAutocompleteService,
                                Place: places_1.MigrationPlace,
                                PlacesService: places_1.MigrationPlacesService,
                                PlacesServiceStatus: googleCommon_1.PlacesServiceStatus,
                                SearchBox: places_1.MigrationSearchBox,
                            });
                            break;
                        case "routes":
                            resolve({
                                DirectionsRenderer: directions_1.MigrationDirectionsRenderer,
                                DirectionsService: directions_1.MigrationDirectionsService,
                                DistanceMatrixService: directions_1.MigrationDistanceMatrixService,
                                DirectionsStatus: googleCommon_1.DirectionsStatus,
                                TravelMode: directions_1.TravelMode,
                                DistanceMatrixElementStatus: directions_1.DistanceMatrixElementStatus,
                                DistanceMatrixStatus: directions_1.DistanceMatrixStatus,
                            });
                            break;
                        case "marker":
                            resolve({
                                AdvancedMarkerElement: markers_1.MigrationMarker,
                                Marker: markers_1.MigrationMarker,
                            });
                            break;
                        default:
                            console.error(`Unsupported library: ${library}`);
                            resolve({});
                            break;
                    }
                });
            },
        },
    };
    if (postMigrationCallback) {
        window[postMigrationCallback]();
    }
};
migrationInit();
