// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
import { SearchPlaceIndexForPositionCommand, } from "@aws-sdk/client-location";
import { GeocoderStatus, LatLngToLngLat, PlacesServiceStatus, MigrationLatLngBounds, } from "./googleCommon";
import { convertAmazonPlaceToGoogle } from "./places";
class MigrationGeocoder {
    geocode(request, callback) {
        const bounds = request.bounds;
        const language = request.language;
        const location = request.location;
        const address = request.address;
        const placeId = request.placeId;
        // These are the only fields we want to return for the GeocoderResult
        const fields = ["formatted_address", "geometry", "place_id"];
        if (location) {
            const lngLat = LatLngToLngLat(location);
            const input = {
                IndexName: this._placeIndexName,
                Position: lngLat,
            };
            if (language) {
                input.Language = language;
            }
            return new Promise((resolve, reject) => {
                const command = new SearchPlaceIndexForPositionCommand(input);
                this._client
                    .send(command)
                    .then((response) => {
                    const googleResults = [];
                    const results = response.Results;
                    if (results.length !== 0) {
                        results.forEach(function (place) {
                            const newPlace = convertAmazonPlaceToGoogle(place, fields, false);
                            googleResults.push(newPlace);
                        });
                    }
                    if (callback) {
                        callback(googleResults, GeocoderStatus.OK);
                    }
                    resolve({
                        results: googleResults,
                    });
                })
                    .catch((error) => {
                    console.error(error);
                    if (callback) {
                        callback(null, GeocoderStatus.UNKNOWN_ERROR);
                    }
                    reject({
                        status: GeocoderStatus.UNKNOWN_ERROR,
                    });
                });
            });
        }
        else if (placeId) {
            return new Promise((resolve, reject) => {
                const request = {
                    placeId: placeId,
                    fields: fields,
                };
                this._placesService.getDetails(request, (result, status) => {
                    if (status == PlacesServiceStatus.OK) {
                        const googleResults = [result];
                        if (callback) {
                            callback(googleResults, GeocoderStatus.OK);
                        }
                        resolve({
                            results: googleResults,
                        });
                    }
                    else {
                        if (callback) {
                            callback(null, GeocoderStatus.UNKNOWN_ERROR);
                        }
                        reject({
                            status: GeocoderStatus.UNKNOWN_ERROR,
                        });
                    }
                });
            });
        }
        else if (address) {
            return new Promise((resolve, reject) => {
                const request = {
                    query: address,
                    fields: fields,
                };
                // TODO: findPlaceFromQuery only supports a locationBias instead of a bounds, so if bounds
                // was specified we will just use the center to act as a locationBias
                if (bounds) {
                    const latLngBounds = new MigrationLatLngBounds(bounds);
                    request.locationBias = latLngBounds.getCenter();
                }
                this._placesService.findPlaceFromQuery(request, (results, status) => {
                    if (status == PlacesServiceStatus.OK) {
                        if (callback) {
                            callback(results, GeocoderStatus.OK);
                        }
                        resolve({
                            results: results,
                        });
                    }
                    else {
                        if (callback) {
                            callback(null, GeocoderStatus.UNKNOWN_ERROR);
                        }
                        reject({
                            status: GeocoderStatus.UNKNOWN_ERROR,
                        });
                    }
                });
            });
        }
    }
}
export { MigrationGeocoder };
